local NameplateThreat = LibStub('AceAddon-3.0'):NewAddon('NameplateThreat', 'AceEvent-3.0')
_G.NameplateThreat = NameplateThreat

---------------   initialize   ---------------

function NameplateThreat:OnInitialize()
   self.debugging = true

   hooksecurefunc('SetCVar', self.SetCVar)
end

function NameplateThreat:OnEnable()
   self:RegisterEvent('PARTY_MEMBERS_CHANGED')
   self:PARTY_MEMBERS_CHANGED()
   self:CheckForNewNameplates()

   if GetCVarBool('nameplateShowEnemies') then
--~       self:RegisterEvent('UNIT_THREAT_LIST_UPDATE', 'THREAT_UPDATE')  -- don't need this accurate, yet
      self:RegisterEvent('UNIT_THREAT_SITUATION_UPDATE', 'THREAT_UPDATE')
   end
end

function NameplateThreat:OnDisable()
end

---------------   Events   ---------------

NameplateThreat.party = {
   'party1target','partypet1target','party2target','partypet2target','focus',
   'party3target','partypet3target','party4target','partypet4target','target','pettarget',
}
NameplateThreat.raid = {
   'raid1target','raidpet1target','raid2target','raidpet2target','raid3target','raidpet3target',
   'raid4target','raidpet4target','raid5target','raidpet5target','raid6target','raidpet6target',
   'raid7target','raidpet7target','raid8target','raidpet8target','raid9target','raidpet9target',
   'raid10target','raidpet10target','raid11target','raidpet11target','raid12target','raidpet12target',
   'raid13target','raidpet13target','raid14target','raidpet14target','raid15target','raidpet15target',
   'raid16target','raidpet16target','raid17target','raidpet17target','raid18target','raidpet18target',
   'raid19target','raidpet19target','raid20target','raidpet20target','raid21target','raidpet21target',
   'raid22target','raidpet22target','raid23target','raidpet23target','raid24target','raidpet24target',
   'raid25target','raidpet25target','raid26target','raidpet26target','raid27target','raidpet27target',
   'raid28target','raidpet28target','raid29target','raidpet29target','raid30target','raidpet30target',
   'raid31target','raidpet31target','raid32target','raidpet32target','raid33target','raidpet33target',
   'raid34target','raidpet34target','raid35target','raidpet35target','raid36target','raidpet36target',
   'raid37target','raidpet37target','raid38target','raidpet38target','raid39target','raidpet39target',
   'raid40target','raidpet40target','target','pettarget','focus',
}
function NameplateThreat:PARTY_MEMBERS_CHANGED()
	if GetNumRaidMembers() > 0 then
      self.group = self.raid
	elseif GetNumPartyMembers() > 0 then
      self.group = self.party
   else
      self.group = nil
      if self.debugging then
         self.group = self.party -- for testing
      end
      return
   end
   if GetCVarBool('nameplateShowEnemies') then
      --in group and nameplates shown so enable events and onevents
   else
      --disable events and onevents
   end
end

NameplateThreat.select = select
NameplateThreat.CreateFrame = CreateFrame
function NameplateThreat:ProcessChildren(s, e, ...)
   local select = self.select
   local CreateFrame = self.CreateFrame
   for i = s, e do
      CreateFrame('Frame', nil, select(i,...), 'NameplateThreatTemplate')
   end
end

NameplateThreat.numWFChildren = 1
NameplateThreat.WorldFrame = WorldFrame
function NameplateThreat:CheckForNewNameplates()
   local WorldFrame = self.WorldFrame
   local n = WorldFrame:GetNumChildren()
   local numWFChildren = self.numWFChildren
   if n > numWFChildren then
      self:ProcessChildren(numWFChildren+1, n-numWFChildren, WorldFrame:GetChildren())
      self.numWFChildren = n
   end
end

NameplateThreat.ipairs = ipairs
NameplateThreat.UnitName = UnitName
function NameplateThreat:THREAT_UPDATE() --event, unitid, ...) --args are useless values
   if not self.group or not GetCVarBool('nameplateShowEnemies') then return end

   self:CheckForNewNameplates()

   -- loop thru all party/raid&pet targets
   local threat = {}
   local found = {}
   local name
   local UnitName = self.UnitName
   for i,unit in self.ipairs(self.group) do
      name = UnitName(unit)
      if name and not found[unit] then
         self:debug(1, name)
         threat[name..UnitHealth(unit)] = unit
         found[unit] = true
      end
   end

   self.threatTable = threat
end

function NameplateThreat.SetCVar(cvar, val)
   if cvar ~= 'nameplateShowEnemies' then return end
   local NameplateThreat = NameplateThreat
   if val == 1 then
      NameplateThreat:RegisterEvent('UNIT_THREAT_SITUATION_UPDATE', 'THREAT_UPDATE')
      NameplateThreat:CheckForNewNameplates()
   else
      NameplateThreat:UnregisterEvent('UNIT_THREAT_SITUATION_UPDATE')
   end
   --enable/disable onevents too
end

-- this is for the nameplate frame's onupdate
function NameplateThreat:ThreatOnUpdate(elapsed) -- only called when in combat
   local t = self.elapsed + elapsed
   if t < 1 then  --for testing, use .1 otherwise
      self.elapsed = t
      return
   end
   self.elapsed = 0

   local threat = self.threat
   local NameplateThreat = self.NameplateThreat
   local threatTable = NameplateThreat.threatTable

   if threatTable then --and self.NameplateThreat.group then
      local _,_,_,_,_,_,name,lvl = self:GetRegions()
      local unit = threatTable[name:GetText()..self.health:GetValue()]
      if unit then
         local isTanking, status, percentage = UnitDetailedThreatSituation('player', unit)
         if percentage and percentage ~= 0 then
            threat.text:SetText(('%d%%'):format(percentage))
            threat.bg:SetVertexColor(GetThreatStatusColor(status))
            return threat:Show()
         end
      end
   end
   threat:Hide()
end

---------------   Core   ---------------

function NameplateThreat:debug(level, ...)
   if self.debugging then
      print(strrep('   ', level), ...)
   end
end
